@@ -1,12 +1,15 @@
 import collections
+import mock
 import threading
+import unittest
 import uuid
 
-from nose.tools import eq_, ok_
+import pytest
 
 from kazoo.exceptions import CancelledError
 from kazoo.exceptions import LockTimeout
 from kazoo.exceptions import NoNodeError
+from kazoo.recipe.lock import Lock
 from kazoo.testing import KazooTestCase
 from kazoo.tests import util as test_util
 
@@ -77,14 +80,14 @@ class KazooLockTests(KazooTestCase):
         try:
             with lock:
                 with self.condition:
-                    eq_(self.active_thread, None)
+                    assert self.active_thread is None
                     self.active_thread = name
                     self.condition.notify_all()
 
                 event.wait()
 
                 with self.condition:
-                    eq_(self.active_thread, name)
+                    assert self.active_thread == name
                     self.active_thread = None
                     self.condition.notify_all()
             self.released.set()
@@ -97,8 +100,10 @@ class KazooLockTests(KazooTestCase):
         lock_name = uuid.uuid4().hex
         lock = self.client.Lock(self.lockpath, lock_name)
         event = self.make_event()
-        thread = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                  args=(lock_name, lock, event))
+        thread = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=(lock_name, lock, event),
+        )
         thread.start()
 
         lock2_name = uuid.uuid4().hex
@@ -107,7 +112,7 @@ class KazooLockTests(KazooTestCase):
         # wait for any contender to show up on the lock
         wait = self.make_wait()
         wait(anotherlock.contenders)
-        eq_(anotherlock.contenders(), [lock_name])
+        assert anotherlock.contenders() == [lock_name]
 
         with self.condition:
             while self.active_thread != lock_name:
@@ -131,8 +136,9 @@ class KazooLockTests(KazooTestCase):
         for name in names:
             e = self.make_event()
             l = self.client.Lock(self.lockpath, name)
-            t = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                 args=(name, l, e))
+            t = self.make_thread(
+                target=self._thread_lock_acquire_til_event, args=(name, l, e)
+            )
             contender_bits[name] = (t, e)
             threads.append(t)
 
@@ -148,7 +154,7 @@ class KazooLockTests(KazooTestCase):
         wait(lambda: len(lock.contenders()) == 6)
         contenders = lock.contenders()
 
-        eq_(contenders[0], "test")
+        assert contenders[0] == "test"
         contenders = contenders[1:]
         remaining = list(contenders)
 
@@ -161,9 +167,9 @@ class KazooLockTests(KazooTestCase):
             with self.condition:
                 while not self.active_thread:
                     self.condition.wait()
-                eq_(self.active_thread, contender)
+                assert self.active_thread == contender
 
-            eq_(lock.contenders(), remaining)
+            assert lock.contenders() == remaining
             remaining = remaining[1:]
 
             event.set()
@@ -176,9 +182,11 @@ class KazooLockTests(KazooTestCase):
 
     def test_lock_reconnect(self):
         event = self.make_event()
-        other_lock = self.client.Lock(self.lockpath, 'contender')
-        thread = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                  args=('contender', other_lock, event))
+        other_lock = self.client.Lock(self.lockpath, "contender")
+        thread = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=("contender", other_lock, event),
+        )
 
         # acquire the lock ourselves first to make the contender line up
         lock = self.client.Lock(self.lockpath, "test")
@@ -188,7 +196,7 @@ class KazooLockTests(KazooTestCase):
         # wait for the contender to line up on the lock
         wait = self.make_wait()
         wait(lambda: len(lock.contenders()) == 2)
-        eq_(lock.contenders(), ['test', 'contender'])
+        assert lock.contenders() == ["test", "contender"]
 
         self.expire_session(self.make_event)
 
@@ -197,7 +205,7 @@ class KazooLockTests(KazooTestCase):
         with self.condition:
             while not self.active_thread:
                 self.condition.wait()
-            eq_(self.active_thread, 'contender')
+            assert self.active_thread == "contender"
 
         event.set()
         thread.join()
@@ -207,8 +215,10 @@ class KazooLockTests(KazooTestCase):
         lock = self.client.Lock(self.lockpath, lock_name)
         event = self.make_event()
 
-        thread = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                  args=(lock_name, lock, event))
+        thread = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=(lock_name, lock, event),
+        )
         thread.start()
 
         lock1 = self.client.Lock(self.lockpath, lock_name)
@@ -218,8 +228,8 @@ class KazooLockTests(KazooTestCase):
             if not self.active_thread:
                 self.condition.wait(5)
 
-        ok_(not lock1.acquire(blocking=False))
-        eq_(lock.contenders(), [lock_name])  # just one - itself
+        assert not lock1.acquire(blocking=False)
+        assert lock.contenders() == [lock_name]  # just one - itself
 
         event.set()
         thread.join()
@@ -227,44 +237,50 @@ class KazooLockTests(KazooTestCase):
     def test_lock_fail_first_call(self):
         event1 = self.make_event()
         lock1 = self.client.Lock(self.lockpath, "one")
-        thread1 = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                   args=("one", lock1, event1))
+        thread1 = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=("one", lock1, event1),
+        )
         thread1.start()
 
         # wait for this thread to acquire the lock
         with self.condition:
             if not self.active_thread:
                 self.condition.wait(5)
-                eq_(self.active_thread, "one")
-        eq_(lock1.contenders(), ["one"])
+                assert self.active_thread == "one"
+        assert lock1.contenders() == ["one"]
         event1.set()
         thread1.join()
 
     def test_lock_cancel(self):
         event1 = self.make_event()
         lock1 = self.client.Lock(self.lockpath, "one")
-        thread1 = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                   args=("one", lock1, event1))
+        thread1 = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=("one", lock1, event1),
+        )
         thread1.start()
 
         # wait for this thread to acquire the lock
         with self.condition:
             if not self.active_thread:
                 self.condition.wait(5)
-                eq_(self.active_thread, "one")
+                assert self.active_thread == "one"
 
         client2 = self._get_client()
         client2.start()
         event2 = self.make_event()
         lock2 = client2.Lock(self.lockpath, "two")
-        thread2 = self.make_thread(target=self._thread_lock_acquire_til_event,
-                                   args=("two", lock2, event2))
+        thread2 = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=("two", lock2, event2),
+        )
         thread2.start()
 
         # this one should block in acquire. check that it is a contender
         wait = self.make_wait()
         wait(lambda: len(lock2.contenders()) > 1)
-        eq_(lock2.contenders(), ["one", "two"])
+        assert lock2.contenders() == ["one", "two"]
 
         lock2.cancel()
         with self.condition:
@@ -272,7 +288,7 @@ class KazooLockTests(KazooTestCase):
                 self.condition.wait()
                 assert "two" in self.cancelled_threads
 
-        eq_(lock2.contenders(), ["one"])
+        assert lock2.contenders() == ["one"]
 
         thread2.join()
         event1.set()
@@ -282,19 +298,19 @@ class KazooLockTests(KazooTestCase):
     def test_lock_no_double_calls(self):
         lock1 = self.client.Lock(self.lockpath, "one")
         lock1.acquire()
-        self.assertTrue(lock1.is_acquired)
-        self.assertFalse(lock1.acquire(timeout=0.5))
-        self.assertTrue(lock1.is_acquired)
+        assert lock1.is_acquired is True
+        assert lock1.acquire(timeout=0.5) is False
+        assert lock1.is_acquired is True
         lock1.release()
-        self.assertFalse(lock1.is_acquired)
+        assert lock1.is_acquired is False
 
     def test_lock_same_thread_no_block(self):
         lock = self.client.Lock(self.lockpath, "one")
         gotten = lock.acquire(blocking=False)
-        self.assertTrue(gotten)
-        self.assertTrue(lock.is_acquired)
+        assert gotten is True
+        assert lock.is_acquired is True
         gotten = lock.acquire(blocking=False)
-        self.assertFalse(gotten)
+        assert gotten is False
 
     def test_lock_many_threads_no_block(self):
         lock = self.client.Lock(self.lockpath, "one")
@@ -313,7 +329,7 @@ class KazooLockTests(KazooTestCase):
             t = threads.pop()
             t.join()
 
-        self.assertEqual(1, sum(list(attempts)))
+        assert sum(list(attempts)) == 1
 
     def test_lock_many_threads(self):
         sleep_func = self.client.handler.sleep_func
@@ -346,8 +362,8 @@ class KazooLockTests(KazooTestCase):
             t = threads.pop()
             t.join()
 
-        self.assertEqual(self.thread_count, len(acquires))
-        self.assertEqual([1] * self.thread_count, list(differences))
+        assert len(acquires) == self.thread_count
+        assert list(differences) == [1] * self.thread_count
 
     def test_lock_reacquire(self):
         lock = self.client.Lock(self.lockpath, "one")
@@ -361,7 +377,7 @@ class KazooLockTests(KazooTestCase):
         client1.start()
         lock = client1.Lock(self.lockpath, "ephemeral")
         lock.acquire(ephemeral=False)
-        znode = self.lockpath + '/' + lock.node
+        znode = self.lockpath + "/" + lock.node
         client1.stop()
         try:
             self.client.get(znode)
@@ -381,7 +397,7 @@ class KazooLockTests(KazooTestCase):
             with lock:
                 started.set()
                 event.wait(timeout)
-                if not event.isSet():
+                if not event.is_set():
                     # Eventually fail to avoid hanging the tests
                     self.fail("lock2 never timed out")
 
@@ -393,7 +409,7 @@ class KazooLockTests(KazooTestCase):
         client2 = self._get_client()
         client2.start()
         started.wait(5)
-        self.assertTrue(started.isSet())
+        assert started.is_set() is True
         lock2 = client2.Lock(self.lockpath, "two")
         try:
             lock2.acquire(timeout=timeout)
@@ -413,37 +429,35 @@ class KazooLockTests(KazooTestCase):
         # Test that we can obtain a read lock
         lock = self.client.ReadLock(self.lockpath, "reader one")
         gotten = lock.acquire(blocking=False)
-        self.assertTrue(gotten)
-        self.assertTrue(lock.is_acquired)
+        assert gotten is True
+        assert lock.is_acquired is True
         # and that it's still not reentrant.
         gotten = lock.acquire(blocking=False)
-        self.assertFalse(gotten)
-
+        assert gotten is False
         # Test that a second client we can share the same read lock
         client2 = self._get_client()
         client2.start()
         lock2 = client2.ReadLock(self.lockpath, "reader two")
         gotten = lock2.acquire(blocking=False)
-        self.assertTrue(gotten)
-        self.assertTrue(lock2.is_acquired)
+        assert gotten is True
+        assert lock2.is_acquired is True
         gotten = lock2.acquire(blocking=False)
-        self.assertFalse(gotten)
-
+        assert gotten is False
         # Test that a writer is unable to share it
         client3 = self._get_client()
         client3.start()
         lock3 = client3.WriteLock(self.lockpath, "writer")
         gotten = lock3.acquire(blocking=False)
-        self.assertFalse(gotten)
+        assert gotten is False
 
     def test_write_lock(self):
         # Test that we can obtain a write lock
         lock = self.client.WriteLock(self.lockpath, "writer")
         gotten = lock.acquire(blocking=False)
-        self.assertTrue(gotten)
-        self.assertTrue(lock.is_acquired)
+        assert gotten is True
+        assert lock.is_acquired is True
         gotten = lock.acquire(blocking=False)
-        self.assertFalse(gotten)
+        assert gotten is False
 
         # Test that we are unable to obtain a read lock while the
         # write lock is held.
@@ -451,11 +465,68 @@ class KazooLockTests(KazooTestCase):
         client2.start()
         lock2 = client2.ReadLock(self.lockpath, "reader")
         gotten = lock2.acquire(blocking=False)
-        self.assertFalse(gotten)
+        assert gotten is False
+
+    def test_rw_lock(self):
+        reader_event = self.make_event()
+        reader_lock = self.client.ReadLock(self. lockpath, "reader")
+        reader_thread = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=("reader", reader_lock, reader_event)
+        )
+
+        writer_event = self.make_event()
+        writer_lock = self.client.WriteLock(self. lockpath, "writer")
+        writer_thread = self.make_thread(
+            target=self._thread_lock_acquire_til_event,
+            args=("writer", writer_lock, writer_event)
+        )
+
+        # acquire a write lock ourselves first to make the others line up
+        lock = self.client.WriteLock(self.lockpath, "test")
+        lock.acquire()
+
+        reader_thread.start()
+        writer_thread.start()
+
+        # wait for everyone to line up on the lock
+        wait = self.make_wait()
+        wait(lambda: len(lock.contenders()) == 3)
+        contenders = lock.contenders()
+
+        assert contenders[0] == "test"
+        remaining = contenders[1:]
+
+        # release the lock and contenders should claim it in order
+        lock.release()
+
+        contender_bits = {
+            "reader": (reader_thread, reader_event),
+            "writer": (writer_thread, writer_event),
+        }
+
+        for contender in ("reader", "writer"):
+            thread, event = contender_bits[contender]
+
+            with self.condition:
+                while not self.active_thread:
+                    self.condition.wait()
+                assert self.active_thread == contender
+
+            assert lock.contenders() == remaining
+            remaining = remaining[1:]
+
+            event.set()
+
+            with self.condition:
+                while self.active_thread:
+                    self.condition.wait()
+
+        reader_thread.join()
+        writer_thread.join()
 
 
 class TestSemaphore(KazooTestCase):
-
     def __init__(self, *args, **kw):
         super(TestSemaphore, self).__init__(*args, **kw)
         self.threads_made = []
@@ -510,37 +581,42 @@ class TestSemaphore(KazooTestCase):
         thread.start()
         started.wait(10)
 
-        self.assertFalse(event.is_set())
+        assert event.is_set() is False
 
         sem1.release()
         event.wait(10)
-        self.assert_(event.is_set())
+        assert event.is_set() is True
         thread.join()
 
     def test_non_blocking(self):
         sem1 = self.client.Semaphore(
-            self.lockpath, identifier='sem1', max_leases=2)
+            self.lockpath, identifier="sem1", max_leases=2
+        )
         sem2 = self.client.Semaphore(
-            self.lockpath, identifier='sem2', max_leases=2)
+            self.lockpath, identifier="sem2", max_leases=2
+        )
         sem3 = self.client.Semaphore(
-            self.lockpath, identifier='sem3', max_leases=2)
+            self.lockpath, identifier="sem3", max_leases=2
+        )
 
         sem1.acquire()
         sem2.acquire()
-        ok_(not sem3.acquire(blocking=False))
-        eq_(set(sem1.lease_holders()), set(['sem1', 'sem2']))
+        assert not sem3.acquire(blocking=False)
+        assert set(sem1.lease_holders()) == set(["sem1", "sem2"])
         sem2.release()
         # the next line isn't required, but avoids timing issues in tests
         sem3.acquire()
-        eq_(set(sem1.lease_holders()), set(['sem1', 'sem3']))
+        assert set(sem1.lease_holders()) == set(["sem1", "sem3"])
         sem1.release()
         sem3.release()
 
     def test_non_blocking_release(self):
         sem1 = self.client.Semaphore(
-            self.lockpath, identifier='sem1', max_leases=1)
+            self.lockpath, identifier="sem1", max_leases=1
+        )
         sem2 = self.client.Semaphore(
-            self.lockpath, identifier='sem2', max_leases=1)
+            self.lockpath, identifier="sem2", max_leases=1
+        )
         sem1.acquire()
         sem2.acquire(blocking=False)
 
@@ -553,7 +629,7 @@ class TestSemaphore(KazooTestCase):
         event = self.make_event()
 
         def sema_one():
-            with self.client.Semaphore(self.lockpath, 'fred', max_leases=1):
+            with self.client.Semaphore(self.lockpath, "fred", max_leases=1):
                 started.set()
                 event.wait()
 
@@ -562,13 +638,13 @@ class TestSemaphore(KazooTestCase):
         started.wait()
         sem1 = self.client.Semaphore(self.lockpath)
         holders = sem1.lease_holders()
-        eq_(holders, ['fred'])
+        assert holders == ["fred"]
         event.set()
         thread.join()
 
     def test_semaphore_cancel(self):
-        sem1 = self.client.Semaphore(self.lockpath, 'fred', max_leases=1)
-        sem2 = self.client.Semaphore(self.lockpath, 'george', max_leases=1)
+        sem1 = self.client.Semaphore(self.lockpath, "fred", max_leases=1)
+        sem2 = self.client.Semaphore(self.lockpath, "george", max_leases=1)
         sem1.acquire()
         started = self.make_event()
         event = self.make_event()
@@ -584,28 +660,29 @@ class TestSemaphore(KazooTestCase):
         thread = self.make_thread(target=sema_one, args=())
         thread.start()
         started.wait()
-        eq_(sem1.lease_holders(), ['fred'])
-        eq_(event.is_set(), False)
+        assert sem1.lease_holders() == ["fred"]
+        assert not event.is_set()
         sem2.cancel()
         event.wait()
-        eq_(event.is_set(), True)
+        assert event.is_set()
         thread.join()
 
     def test_multiple_acquire_and_release(self):
-        sem1 = self.client.Semaphore(self.lockpath, 'fred', max_leases=1)
+        sem1 = self.client.Semaphore(self.lockpath, "fred", max_leases=1)
         sem1.acquire()
         sem1.acquire()
 
-        eq_(True, sem1.release())
-        eq_(False, sem1.release())
+        assert sem1.release()
+        assert not sem1.release()
 
     def test_handle_session_loss(self):
-        expire_semaphore = self.client.Semaphore(self.lockpath, 'fred',
-                                                 max_leases=1)
+        expire_semaphore = self.client.Semaphore(
+            self.lockpath, "fred", max_leases=1
+        )
 
         client = self._get_client()
         client.start()
-        lh_semaphore = client.Semaphore(self.lockpath, 'george', max_leases=1)
+        lh_semaphore = client.Semaphore(self.lockpath, "george", max_leases=1)
         lh_semaphore.acquire()
 
         started = self.make_event()
@@ -622,7 +699,7 @@ class TestSemaphore(KazooTestCase):
         thread1.start()
 
         started.wait()
-        eq_(lh_semaphore.lease_holders(), ['george'])
+        assert lh_semaphore.lease_holders() == ["george"]
 
         # Fired in a separate thread to make sure we can see the effect
         expired = self.make_event()
@@ -640,7 +717,7 @@ class TestSemaphore(KazooTestCase):
         client.stop()
 
         event.wait(15)
-        eq_(expire_semaphore.lease_holders(), ['fred'])
+        assert expire_semaphore.lease_holders() == ["fred"]
         event2.set()
 
         for t in (thread1, thread2):
@@ -651,18 +728,19 @@ class TestSemaphore(KazooTestCase):
         sem2 = self.client.Semaphore(self.lockpath, max_leases=2)
 
         sem1.acquire()
-        self.assertRaises(ValueError, sem2.acquire)
+        with pytest.raises(ValueError):
+            sem2.acquire()
 
     def test_inconsistent_max_leases_other_data(self):
         sem1 = self.client.Semaphore(self.lockpath, max_leases=1)
         sem2 = self.client.Semaphore(self.lockpath, max_leases=2)
 
         self.client.ensure_path(self.lockpath)
-        self.client.set(self.lockpath, b'a$')
+        self.client.set(self.lockpath, b"a$")
 
         sem1.acquire()
         # sem2 thinks it's ok to have two lease holders
-        ok_(sem2.acquire(blocking=False))
+        assert sem2.acquire(blocking=False)
 
     def test_reacquire(self):
         lock = self.client.Semaphore(self.lockpath)
@@ -673,11 +751,11 @@ class TestSemaphore(KazooTestCase):
 
     def test_acquire_after_cancelled(self):
         lock = self.client.Semaphore(self.lockpath)
-        self.assertTrue(lock.acquire())
-        self.assertTrue(lock.release())
+        assert lock.acquire() is True
+        assert lock.release() is True
         lock.cancel()
-        self.assertTrue(lock.cancelled)
-        self.assertTrue(lock.acquire())
+        assert lock.cancelled is True
+        assert lock.acquire() is True
 
     def test_timeout(self):
         timeout = 3
@@ -692,7 +770,7 @@ class TestSemaphore(KazooTestCase):
             with sem:
                 started.set()
                 event.wait(timeout)
-                if not event.isSet():
+                if not event.is_set():
                     # Eventually fail to avoid hanging the tests
                     self.fail("sem2 never timed out")
 
@@ -704,7 +782,7 @@ class TestSemaphore(KazooTestCase):
         client2 = self._get_client()
         client2.start()
         started.wait(5)
-        self.assertTrue(started.isSet())
+        assert started.is_set() is True
         sem2 = client2.Semaphore(self.lockpath, "two")
         try:
             sem2.acquire(timeout=timeout)
@@ -716,3 +794,28 @@ class TestSemaphore(KazooTestCase):
             # Cleanup
             t.join()
             client2.stop()
+
+
+class TestSequence(unittest.TestCase):
+    def test_get_predecessor(self):
+        """Validate selection of predecessors.
+        """
+        goLock = "_c_8eb60557ba51e0da67eefc47467d3f34-lock-0000000031"
+        pyLock = "514e5a831836450cb1a56c741e990fd8__lock__0000000032"
+        children = ["hello", goLock, "world", pyLock]
+        client = mock.MagicMock()
+        client.get_children.return_value = children
+        lock = Lock(client, "test")
+        assert lock._get_predecessor(pyLock) is None
+
+    def test_get_predecessor_go(self):
+        """Test selection of predecessor when instructed to consider go-zk
+        locks.
+        """
+        goLock = "_c_8eb60557ba51e0da67eefc47467d3f34-lock-0000000031"
+        pyLock = "514e5a831836450cb1a56c741e990fd8__lock__0000000032"
+        children = ["hello", goLock, "world", pyLock]
+        client = mock.MagicMock()
+        client.get_children.return_value = children
+        lock = Lock(client, "test", extra_lock_patterns=["-lock-"])
+        assert lock._get_predecessor(pyLock) == goLock
